#include <stdint.h>
#include <stdio.h>
#include <curl/curl.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <libconfig.h>

#include "led.h"


const char INFLUXURL_KEY[] = "influxUrl";
const char DEFAULT_INFLUXURL[] = "http://172.16.3.15:8086/write?db=smarthome2&precision=ms";
const char *influxUrl;
const char INFLUXUSER_KEY[] = "influxUser";
const char *influxUser;
const char INFLUXPASS_KEY[] = "influxPass";
const char *influxPass;
const char INFLUXTAG_KEY[] = "influxTag";
const char *influxTag;

extern uint32_t skipped;


// #define BUFSIZE 131070
#define BUFSIZE 65535
// #define BUFSIZE 1024
char influxBuffer[BUFSIZE];
char *bufferNextEntry;

#define HOSTNAMESIZE 128
char hostname[HOSTNAMESIZE];

static void influxClearBuffer() {
    memset(influxBuffer, 0, BUFSIZE);
    bufferNextEntry = influxBuffer;

}

void influxInit(config_t *pCfg) {
    if (! config_lookup_string(pCfg, INFLUXURL_KEY, &influxUrl)) {
        influxUrl = DEFAULT_INFLUXURL;
    }
    if (! config_lookup_string(pCfg, INFLUXUSER_KEY, &influxUser)) {
        influxUser = NULL;
    }
    if (! config_lookup_string(pCfg, INFLUXPASS_KEY, &influxPass)) {
        influxPass = NULL;
    }
    if (! config_lookup_string(pCfg, INFLUXTAG_KEY, &influxTag)) {
        gethostname(hostname, HOSTNAMESIZE);
        influxTag = hostname;
    }
    influxClearBuffer();
}



static void influxSendRequest() {
    led(E_RED, false);
    led(E_BLUE, true);
    CURL *curl = curl_easy_init();
    if(curl) {
        curl_easy_setopt(curl, CURLOPT_URL, influxUrl);
        if (influxUser && influxPass) {
            curl_easy_setopt(curl, CURLOPT_HTTPAUTH, CURLAUTH_DIGEST);
            curl_easy_setopt(curl, CURLOPT_USERNAME, influxUser);
            curl_easy_setopt(curl, CURLOPT_PASSWORD, influxPass);
        }
        curl_easy_setopt(curl, CURLOPT_POSTFIELDS, influxBuffer);
        CURLcode res = curl_easy_perform(curl);
        if(res != CURLE_OK) {
            fprintf(stderr, "curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
            led(E_RED, true);
        }
        curl_easy_cleanup(curl);
    }
    led(E_BLUE, false);
}

void influxAddFrequency(double f) {
    static uint32_t entries = 0;
    static uint32_t totalEntries = 0;
    char tmpBuf[256];
    struct timespec t;

    clock_gettime(CLOCK_REALTIME, &t);
    uint64_t tt = (((uint64_t)t.tv_sec) * 1000) + (((uint64_t)t.tv_nsec) / 1000000);
    int c = sprintf(tmpBuf, "mainsfrequency,host=%s freq=%f %llu\n", influxTag, f, tt);

    if ((bufferNextEntry + c + 10) > (influxBuffer + BUFSIZE)) {
        influxSendRequest();
        influxClearBuffer();
        totalEntries += entries;
        fprintf(stderr, "%u entries sent to database, in total %u, skipped: %u\n", entries, totalEntries, skipped);
        entries = 0;
    }

    memcpy(bufferNextEntry, tmpBuf, c);
    bufferNextEntry += c;
    entries++;
}


